﻿using System.Windows;
using System.Windows.Controls;
using CommunityToolkit.Mvvm.ComponentModel;
using LibS7Adv;
using CtrEditor.Simulacion;
using Xceed.Wpf.Toolkit.PropertyGrid.Attributes;
using System.ComponentModel;
using CtrEditor.FuncionesBase;
using System.Text.Json.Serialization;

namespace CtrEditor.ObjetosSim
{
    /// <summary>
    /// Interaction logic for ucTransporteTTop.xaml
    /// </summary>
    /// 

    public partial class osTransporteTTop : osBase, IosBase
    {

        private simTransporte SimGeometria;
        private osVMmotorSim Motor;

        public static string NombreClase()
        {
            return "Transporte TTOP";
        }
        private string nombre = NombreClase();

        [property: Category("Identificación")]
        [property: Description("Nombre identificativo del objeto")]
        [property: Name("Nombre")]
        public override string Nombre
        {
            get => nombre;
            set => SetProperty(ref nombre, value);
        }

        [ObservableProperty]
        [property: Category("Simulación")]
        [property: Description("Velocidad actual del transporte en m/s")]
        [property: Name("Velocidad Actual")]
        public float velocidadActual;

        partial void OnVelocidadActualChanged(float value)
        {
            SetSpeed();
        }

        [ObservableProperty]
        [property: Category("Configuración")]
        [property: Description("Invierte el sentido de movimiento del transporte")]
        [property: Name("Invertir Dirección")]
        bool invertirDireccion;

        partial void OnInvertirDireccionChanged(bool value)
        {
            SetSpeed();
            if (_visualRepresentation is ucTransporteTTop uc)
            {
                CrearAnimacionStoryBoardTrasnporte(uc.Transporte, InvertirDireccion);
                ActualizarAnimacionStoryBoardTransporte(VelocidadActual);
            }
        }

        void SetSpeed()
        {
            if (InvertirDireccion)
                SimGeometria?.SetSpeed(-VelocidadActual);
            else
                SimGeometria?.SetSpeed(VelocidadActual);
            ActualizarAnimacionStoryBoardTransporte(VelocidadActual);
        }

        [ObservableProperty]
        [property: Description("Tag de bit para activar enlace con motor")]
        [property: Category("Enlace PLC")]
        [property: Name("Tag Activación Motor")]
        string tag_ReleActivatedMotor;

        [ObservableProperty]
        [property: Description("Seleccionar motor para enlazar")]
        [property: Category("Enlace PLC")]
        [property: Name("Motor Enlazado")]
        [property: ItemsSource(typeof(osBaseItemsSource<osVMmotorSim>))]
        string id_Motor;

        [JsonIgnore]
        private PropertyChangedEventHandler motorPropertyChangedHandler;

        partial void OnId_MotorChanged(string value)
        {
            if (Motor != null && motorPropertyChangedHandler != null)
                Motor.PropertyChanged -= motorPropertyChangedHandler;

            if (_mainViewModel != null && !string.IsNullOrEmpty(value))
            {
                Motor = (osVMmotorSim)_mainViewModel.ObjetosSimulables.FirstOrDefault(s => s is osVMmotorSim motor && motor.Nombre == value);
                if (Motor != null)
                {
                    motorPropertyChangedHandler = (sender, e) =>
                    {
                        if (e.PropertyName == nameof(osVMmotorSim.Nombre))
                        {
                            Id_Motor = ((osVMmotorSim)sender).Nombre;
                        }
                    };
                    Motor.PropertyChanged += motorPropertyChangedHandler;
                }
            }
        }


        [ObservableProperty]
        [property: Category("Configuración")]
        [property: Description("Coeficiente de fricción entre objetos y superficie")]
        [property: Name("Coeficiente de Fricción")]
        public float frictionCoefficient;

        [ObservableProperty]
        [property: Category("Configuración")]
        [property: Description("Velocidad máxima a 50Hz")]
        [property: Name("Velocidad Máxima 50Hz")]
        public float velMax50hz;

        [ObservableProperty]
        [property: Category("Configuración")]
        [property: Description("Tiempo de rampa para acelerar/desacelerar")]
        [property: Name("Tiempo de Rampa")]
        public float tiempoRampa;

        [ObservableProperty]
        [property: Category("Información")]
        [property: Description("Indica si el transporte está en marcha")]
        [property: Name("En Marcha")]
        public bool esMarcha;


        private void ActualizarGeometrias()
        {
            if (_visualRepresentation is ucTransporteTTop uc)
            {
                UpdateRectangle(SimGeometria, uc.Transporte, Alto, Ancho, Angulo);
                SetSpeed();
            }
            ActualizarAnimacionStoryBoardTransporte(VelocidadActual);
        }

        public override void OnMoveResizeRotate()
        {
            ActualizarGeometrias();
        }

        public osTransporteTTop()
        {
            Ancho = 1;
            Alto = 0.10f;
            Tag_ReleActivatedMotor = "1";
        }

        public override void SimulationStop()
        {
            // Se llama al detener la simulacion
            ActualizarAnimacionStoryBoardTransporte(VelocidadActual);
        }
        public override void UpdateGeometryStart()
        {
            // Se llama antes de la simulacion
            ActualizarGeometrias();
        }

        public override void UpdatePLC(PLCViewModel plc, int elapsedMilliseconds)
        {
            if (Motor != null)
                if (Motor is osVMmotorSim motor)
                    if (LeerBitTag(Tag_ReleActivatedMotor))
                        VelocidadActual = motor.Velocidad;
                    else
                        VelocidadActual = 0;
        }

        public override void ucLoaded()
        {
            // El UserControl ya se ha cargado y podemos obtener las coordenadas para 
            // crear el objeto de simulacion
            base.ucLoaded();
            OnId_MotorChanged(Id_Motor); // Link Id_Motor = Motor

            if (_visualRepresentation is ucTransporteTTop uc)
            {
                try
                {
                    // Asegurar que el layout esté actualizado antes de crear la geometría
                    uc.UpdateLayout();

                    // Validar que el rectángulo esté disponible y tenga dimensiones válidas
                    if (uc.Transporte != null &&
                        (!double.IsNaN(uc.Transporte.ActualWidth) && uc.Transporte.ActualWidth > 0) ||
                        (!double.IsNaN(uc.Transporte.Width) && uc.Transporte.Width > 0))
                    {
                        SimGeometria = AddRectangle(simulationManager, uc.Transporte, Alto, Ancho, Angulo);
                        CrearAnimacionStoryBoardTrasnporte(uc.Transporte, InvertirDireccion);
                    }
                    else
                    {
                        // Si el rectángulo no está listo, intentar después del próximo layout
                        uc.Dispatcher.BeginInvoke(new Action(() =>
                        {
                            if (uc.Transporte != null && simulationManager != null)
                            {
                                SimGeometria = AddRectangle(simulationManager, uc.Transporte, Alto, Ancho, Angulo);
                                CrearAnimacionStoryBoardTrasnporte(uc.Transporte, InvertirDireccion);
                            }
                        }), System.Windows.Threading.DispatcherPriority.Loaded);
                    }
                }
                catch (Exception ex)
                {
                    // Log del error para diagnóstico pero continuar sin fallar
                    System.Diagnostics.Debug.WriteLine($"Error al crear geometría de simulación: {ex.Message}");

                    // Intentar crear la geometría más tarde
                    uc.Dispatcher.BeginInvoke(new Action(() =>
                    {
                        try
                        {
                            if (uc.Transporte != null && simulationManager != null)
                            {
                                SimGeometria = AddRectangle(simulationManager, uc.Transporte, Alto, Ancho, Angulo);
                                CrearAnimacionStoryBoardTrasnporte(uc.Transporte, InvertirDireccion);
                            }
                        }
                        catch (Exception ex2)
                        {
                            System.Diagnostics.Debug.WriteLine($"Segundo intento falló: {ex2.Message}");
                        }
                    }), System.Windows.Threading.DispatcherPriority.ApplicationIdle);
                }
            }
        }
        public override void ucUnLoaded()
        {
            // El UserControl se esta eliminando
            // eliminar el objeto de simulacion
            simulationManager.Remove(SimGeometria);
        }

    }

    public partial class ucTransporteTTop : UserControl, IDataContainer
    {
        public osBase? Datos { get; set; }
        public int zIndex_fromFrames { get; set; }

        public ucTransporteTTop()
        {
            InitializeComponent();
            this.Loaded += OnLoaded;
            this.Unloaded += OnUnloaded;
        }
        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucLoaded();
        }
        private void OnUnloaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucUnLoaded();
        }
        public void Highlight(bool State) { }
        public ZIndexEnum ZIndex_Base()
        {
            return ZIndexEnum.Estaticos;
        }


    }


}



